-- top module
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity RGBCONV is
	port (
		TP1				: out   std_logic;						-- TEST port
		LEDGN			: out   std_logic_vector(2 downto 1);	-- LED
		CLK1			: in    std_logic;						-- clock input #1
		CLK2			: in    std_logic;						-- clock input #2
		GPIO_D			: inout std_logic_vector(58 downto 1)	-- GPIO data
	);
end RGBCONV;

-- CN2
-- A01:CLK1        B01:GPIO_D[ 1]
-- A02:GPIO_D[ 2]  B02:GPIO_D[ 3]
-- A03:GPIO_D[ 4]  B03:GPIO_D[ 5]
-- A04:GPIO_D[ 6]  B04:GPIO_D[ 7]
-- A05:GPIO_D[ 8]  B05:GPIO_D[ 9]
-- A06:3.3V        B06:3.3V
-- A07:GPIO_D[10]  B07:GPIO_D[11]
-- A08:GPIO_D[12]  B08:GPIO_D[13]
-- A09:GPIO_D[14]  B09:GPIO_D[15]
-- A10:GPIO_D[16]  B10:GPIO_D[17]
-- A11:GND         B11:GND
-- A12:GPIO_D[18]  B12:GPIO_D[19]
-- A13:GPIO_D[20]  B13:GPIO_D[21]
-- A14:GPIO_D[22]  B14:GPIO_D[23]
-- A15:GPIO_D[24]  B15:GPIO_D[25]
-- A16:GPIO_D[26]  B16:GPIO_D[27]
-- A17:GPIO_D[28]  B17:GPIO_D[29]

-- CN3
-- A01:CLK2        B01:GPIO_D[30]
-- A02:GPIO_D[31]  B02:GPIO_D[32]
-- A03:GPIO_D[33]  B03:GPIO_D[34]
-- A04:GPIO_D[35]  B04:GPIO_D[36]
-- A05:GPIO_D[37]  B05:GPIO_D[38]
-- A06:3.3V        B06:3.3V
-- A07:GPIO_D[39]  B07:GPIO_D[40]
-- A08:GPIO_D[41]  B08:GPIO_D[42]
-- A09:GPIO_D[43]  B09:GPIO_D[44]
-- A10:GPIO_D[45]  B10:GPIO_D[46]
-- A11:GND         B11:GND
-- A12:GPIO_D[47]  B12:GPIO_D[48]
-- A13:GPIO_D[49]  B13:GPIO_D[50]
-- A14:GPIO_D[51]  B14:GPIO_D[52]
-- A15:GPIO_D[53]  B15:GPIO_D[54]
-- A16:GPIO_D[55]  B16:GPIO_D[56]
-- A17:GPIO_D[57]  B17:GPIO_D[58]


architecture RTL of RGBCONV is

	component CLKGEN is
		port (
			CLK50M		: in  std_logic;
			RSTN		: in  std_logic;
			CLK100MOUT	: out std_logic;
			CLK25MOUT	: out std_logic;
			LOCK_PLL	: out std_logic
		);
	end component;

	component P6SYNC is
		port (
			P6_HSYNCN	: in  std_logic;
			P6_VSYNCN	: in  std_logic;
			P6_RGBH		: in  std_logic_vector(3 downto 0);
			P6_CLK		: in  std_logic;
			CLK100M		: in  std_logic;
			RSTN		: in  std_logic;
			SRMODE		: out std_logic;
			CLKENB		: out std_logic;
			DT_HSYNCN	: out std_logic;
			DT_VSYNCN	: out std_logic;
			DT_RGBH		: out std_logic_vector(3 downto 0);
			SFTDT_U		: out std_logic_vector(2 downto 0);
			SFTDT_D		: out std_logic_vector(2 downto 0);
			SFTHS_U		: out std_logic_vector(2 downto 0);
			SFTHS_D		: out std_logic_vector(2 downto 0);
			LOCK1		: out std_logic;
			LOCK2		: out std_logic
		);
	end component;

	component DATASFT is
		port (
			HSYNCN		: in  std_logic;
			VSYNCN		: in  std_logic;
			RGBH		: in  std_logic_vector(3 downto 0);
			CLKENB		: in  std_logic;
			SRMODE		: in  std_logic;
			LOCK1		: in  std_logic;
			LOCK2		: in  std_logic;
			CLK100M		: in  std_logic;
			RSTN		: in  std_logic;
			HSYNCN_O	: out std_logic;
			VSYNCN_O	: out std_logic;
			RGBH_O		: out std_logic_vector(3 downto 0);
			LOCK3		: out std_logic;
			LOCK4		: out std_logic
		);
	end component;

	component VGAOUT is
		port (
			DISPD		: in  std_logic_vector(3 downto 0);
			DISPTMG_HS	: in  std_logic;
			DISPTMG_VS	: in  std_logic;
			CLKENB		: in  std_logic;
			SRMODE		: in  std_logic;
			SC4COLORON	: in  std_logic;
			SC4COLORMD	: in  std_logic_vector(1 downto 0);
			LOCK_PLL	: in  std_logic;
			CLK100M		: in  std_logic;
			CLK25M		: in  std_logic;
			RSTN		: in  std_logic;
			VGA_R		: out std_logic_vector(3 downto 0);
			VGA_G		: out std_logic_vector(3 downto 0);
			VGA_B		: out std_logic_vector(3 downto 0);
			VGA_HS		: out std_logic;
			VGA_VS		: out std_logic
		);
	end component;

	component COLSEL is
		port (
			MDSEL		: in  std_logic;
			LOCK_PLL	: in  std_logic;
			CLK25M		: in  std_logic;
			RSTN		: in  std_logic;
			COLORMD		: out std_logic_vector(2 downto 0)
		);
	end component;

-- PC-6001 input
	signal p6_r			: std_logic;
	signal p6_g			: std_logic;
	signal p6_b			: std_logic;
	signal p6_h			: std_logic;
	signal p6_hsyncn	: std_logic;
	signal p6_vsyncn	: std_logic;
	signal p6_rgbh		: std_logic_vector(3 downto 0);
	signal p6_clk		: std_logic;

-- clkgen
	signal rstn			: std_logic;
	signal clk100m		: std_logic;
	signal clk25m		: std_logic;
	signal lock_pll		: std_logic;

-- p6sync
	signal sy_hsyncn	: std_logic;
	signal sy_vsyncn	: std_logic;
	signal sy_srmode	: std_logic;
	signal sy_clkenb	: std_logic;
	signal sy_rgbh		: std_logic_vector(3 downto 0);
	signal sy_sftdt_u	: std_logic_vector(2 downto 0);
	signal sy_sftdt_d	: std_logic_vector(2 downto 0);
	signal sy_sfths_u	: std_logic_vector(2 downto 0);
	signal sy_sfths_d	: std_logic_vector(2 downto 0);
	signal sy_lock1		: std_logic;
	signal sy_lock2		: std_logic;

-- datasft
	signal dt_hsyncn	: std_logic;
	signal dt_vsyncn	: std_logic;
	signal dt_rgbh		: std_logic_vector(3 downto 0);
	signal dt_lock3		: std_logic;
	signal dt_lock4		: std_logic;

-- vgaout
	signal vga_r_i		: std_logic_vector(3 downto 0);
	signal vga_g_i		: std_logic_vector(3 downto 0);
	signal vga_b_i		: std_logic_vector(3 downto 0);
	signal vga_hs_i		: std_logic;
	signal vga_vs_i		: std_logic;

-- colormd
	signal mdsel		: std_logic;
	signal colormd		: std_logic_vector(2 downto 0);



-- prob
	signal probdat1		: std_logic_vector(7 downto 0);
	signal probdat2		: std_logic_vector(15 downto 0);
	signal probdat3		: std_logic_vector(7 downto 0);

	signal gpio0_d_tmp		: std_logic_vector(31 downto 0);
	signal gpio0_clkout_tmp	: std_logic_vector(1 downto 0);

	signal gpio1_d_tmp		: std_logic_vector(31 downto 0);
	signal gpio1_clkout_tmp	: std_logic_vector(1 downto 0);

begin

-- clock generate
	rstn <= '1';

	U_CLKGEN : CLKGEN
	port map (
		CLK50M		=> CLK1,		-- 25MHz input
		RSTN		=> rstn,
		CLK100MOUT	=> clk100m,
		CLK25MOUT	=> clk25m,
		LOCK_PLL	=> lock_pll
	);


-- clock detect and data latch

	p6_rgbh <= p6_h & p6_g & p6_r & p6_b;

	U_P6SYNC : P6SYNC
	port map (
		P6_HSYNCN	=> p6_hsyncn,
		P6_VSYNCN	=> p6_vsyncn,
		P6_RGBH		=> p6_rgbh,
		P6_CLK		=> p6_clk,
		CLK100M		=> clk100m,
		RSTN		=> rstn,
		SRMODE		=> sy_srmode,
		CLKENB		=> sy_clkenb,
		DT_HSYNCN	=> sy_hsyncn,
		DT_VSYNCN	=> sy_vsyncn,
		DT_RGBH		=> sy_rgbh,
		SFTDT_U		=> open,
		SFTDT_D		=> open,
		SFTHS_U		=> open,
		SFTHS_D		=> open,
		LOCK1		=> sy_lock1,
		LOCK2		=> sy_lock2
	);

-- data shift
	U_DATASFT : DATASFT
	port map (
		HSYNCN		=> sy_hsyncn,
		VSYNCN		=> sy_vsyncn,
		RGBH		=> sy_rgbh,
		CLKENB		=> sy_clkenb,
		SRMODE		=> sy_srmode,
		LOCK1		=> sy_lock1,
		LOCK2		=> sy_lock2,
		CLK100M		=> clk100m,
		RSTN		=> rstn,
		HSYNCN_O	=> dt_hsyncn,
		VSYNCN_O	=> dt_vsyncn,
		RGBH_O		=> dt_rgbh,
		LOCK3		=> dt_lock3,
		LOCK4		=> dt_lock4
	);


-- VGA output
	U_VGAOUT : VGAOUT
	port map (
		DISPD		=> dt_rgbh,
		DISPTMG_HS	=> dt_hsyncn,
		DISPTMG_VS	=> dt_vsyncn,
		CLKENB		=> sy_clkenb,
		SRMODE		=> sy_srmode,
		SC4COLORON	=> colormd(2),
		SC4COLORMD	=> colormd(1 downto 0),
		LOCK_PLL	=> lock_pll,
		CLK100M		=> clk100m,
		CLK25M		=> clk25m,
		RSTN		=> rstn,
		VGA_R		=> vga_r_i,
		VGA_G		=> vga_g_i,
		VGA_B		=> vga_b_i,
		VGA_HS		=> vga_hs_i,
		VGA_VS		=> vga_vs_i
	);

	GPIO_D(23) <= vga_r_i(0);
	GPIO_D(21) <= vga_r_i(1);
	GPIO_D(19) <= vga_r_i(2);
	GPIO_D(17) <= vga_r_i(3);

	GPIO_D(15) <= vga_g_i(0);
	GPIO_D(13) <= vga_g_i(1);
	GPIO_D(11) <= vga_g_i(2);
	GPIO_D(9)  <= vga_g_i(3);

	GPIO_D(7)  <= vga_b_i(0);
	GPIO_D(5)  <= vga_b_i(1);
	GPIO_D(3)  <= vga_b_i(2);
	GPIO_D(1)  <= vga_b_i(3);

	GPIO_D(34) <= vga_vs_i;
	GPIO_D(36) <= vga_hs_i;


-- color mode select

	mdsel <= not GPIO_D(32);

	U_COLSEL : COLSEL
	port map (
		MDSEL		=> mdsel,
		LOCK_PLL	=> lock_pll,
		CLK25M		=> clk25m,
		RSTN		=> rstn,
		COLORMD		=> colormd
	);


	p6_h      <= GPIO_D(40);
	p6_b      <= GPIO_D(42);
	p6_g      <= GPIO_D(44);
	p6_r      <= GPIO_D(46);
	p6_hsyncn <= GPIO_D(48);
	p6_vsyncn <= GPIO_D(50);
	p6_clk    <= GPIO_D(52);


	LEDGN(1) <= not colormd(2);
	LEDGN(2) <= not (dt_lock3 or dt_lock4);


-- unused pin

	TP1 <= '0';

	GPIO_D( 2) <= 'Z';
	GPIO_D( 4) <= 'Z';
	GPIO_D( 6) <= 'Z';
	GPIO_D( 8) <= 'Z';
	GPIO_D(10) <= 'Z';
	GPIO_D(12) <= 'Z';
	GPIO_D(14) <= 'Z';
	GPIO_D(16) <= 'Z';
	GPIO_D(18) <= 'Z';
	GPIO_D(20) <= 'Z';
	GPIO_D(22) <= 'Z';
	GPIO_D(29 downto 24) <= (others => 'Z');

	GPIO_D(30) <= 'Z';
	GPIO_D(31) <= 'Z';
	GPIO_D(33) <= 'Z';
	GPIO_D(35) <= 'Z';
	GPIO_D(37) <= 'Z';
	GPIO_D(38) <= 'Z';
	GPIO_D(39) <= 'Z';
	GPIO_D(41) <= 'Z';
	GPIO_D(43) <= 'Z';
	GPIO_D(45) <= 'Z';
	GPIO_D(47) <= 'Z';
	GPIO_D(49) <= 'Z';
	GPIO_D(51) <= 'Z';
	GPIO_D(58 downto 53) <= (others => 'Z');


end RTL;
